from Tkinter import *
import main, data, random

class Application(Frame):
    # generate graph helper
    def gen_graph(self):
        self.canvas.delete(ALL)
        print "\n\nGenerate!\n\n"

        h = data.DBHandler()

        filename = self.text.get()
        print "Loading database from " +filename +".\n"

        if (filename.endswith('.txt')):
            h.loadTextDB(filename)
            self.nodeDB = h.getDB()
            self.create_graph()
        elif (filename.endswith('.ser')):
            h.loadSerDB(filename)
            self.nodeDB = h.getDB()
            self.create_graph()
        else:
            print "Error. Re-enter a filename that ends in .ser or .txt."

    # graphs the node database
    def create_graph(self):
        # default scaling values
        self.scale = 30
        self.buf = 20

        maxX = 0
        maxY = 0
        for node in self.nodeDB.getDB():
            n = self.nodeDB.get(node)
            if maxX < n.getXY()[0]:
                maxX = n.getXY()[0]
            if maxY < n.getXY()[1]:
                maxY = n.getXY()[1]

        # scale node sizes based on window size and maxX, maxY
        if (maxX < maxY):
            self.scale = 500 / maxY
        else:
            self.scale = 500 / maxX

        for node in self.nodeDB.getDB():
            n = self.nodeDB.get(node)
            x = n.getXY()[0] * self.scale
            y = n.getXY()[1] * self.scale *0.8

            self.canvas.create_oval(self.buf+x, self.buf+y, self.buf+x+self.scale, self.buf+y+self.scale, outline='white', activefill='blue', width=2)
            self.canvas.create_text(self.buf+x+self.scale/3, self.buf+y+self.scale/3, text=node, font='legend', fill='yellow')
            self.canvas.create_text(self.buf+x+self.scale/3*2, self.buf+y+self.scale/3*2, text=str(n.numChildren()), fill='grey')
            
            for neighbor in n.getNeighbors().keys():
                xx = self.nodeDB.get(neighbor).getXY()[0] * self.scale
                yy = self.nodeDB.get(neighbor).getXY()[1] * self.scale *0.8
                self.canvas.create_line(self.buf+xx+self.scale/2, self.buf+yy+self.scale/2, self.buf+x+self.scale/2, self.buf+y+self.scale/2, fill='white')
                halfx = (self.buf+xx+self.scale/2 + self.buf+x+self.scale/2) /2
                halfy = (self.buf+yy+self.scale/2 + self.buf+y+self.scale/2) /2
                self.canvas.create_text(halfx, halfy, text=n.getNeighbors()[neighbor], fill='green')

    # generate solution helper
    def gen_solution(self):
        try:
            m = main.MainC()
            m.loadDB(self.nodeDB)
            solution = m.run()

            self.create_solution(solution)
        except AttributeError:
            print "Graph node database first."

    # graphs the solution lines on top of the existing graph
    # needs implementation of using different colors for 2+ buses
    def create_solution(self, solution):
        xx = 0
        yy = 0
        color = 'red'
        for node in solution[0]:
            n = self.nodeDB.get(node)
            x = n.getXY()[0] * self.scale
            y = n.getXY()[1] * self.scale *0.8
            
            if (xx != 0 or yy != 0):
                self.canvas.create_line(self.buf+x+self.scale/2, self.buf+y+self.scale/2.1, self.buf+xx+self.scale/2, self.buf+yy+self.scale/2.1, fill=color, width=1, arrow=BOTH)
            xx = x
            yy = y

            if (n == 'school'):
                i = random.randint(0,4)
                if i is 0:
                    color = 'red'
                elif i is 1:
                    color = 'purple'
                elif i is 2:
                    color = 'blue'
                elif i is 3:
                    color = 'orange'
                elif i is 4:
                    color = 'yellow'
                    
    # initializes buttons, canvas
    def createWidgets(self):        
        self.QUIT = Button(self)
        self.QUIT["text"] = "QUIT"
        self.QUIT["fg"]   = "red"
        self.QUIT["command"] =  self.quit

        self.QUIT.pack({"side": "bottom"})

        self.genGraph = Button(self)
        self.genGraph["text"] = "GRAPH",
        self.genGraph["command"] = self.gen_graph
        self.genSol = Button(self)
        self.genSol["text"] = "SOLUTION",
        self.genSol["command"] = self.gen_solution

        self.genGraph.pack({"side": "bottom"})
        self.genSol.pack({"side": "bottom"})
        
        self.text = Entry(self)
        
        self.text.pack({"side": "bottom"})

        self.canvas = Canvas(self, width=600, height=600, background="black")
        self.canvas.pack()

    # constructor
    def __init__(self, master=None):
        Frame.__init__(self, master)

        self.pack()
        self.createWidgets()

root = Tk()
app = Application(master=root)
app.master.title("Bus Routes")
app.master.maxsize(600,600)
app.mainloop()
root.destroy()
